﻿using System;
using System.Collections.Generic;
using System.Linq;
using AT.SampleApp.Cqrs.Aggregate;
using AT.SampleApp.Cqrs.Command;
using AT.SampleApp.Cqrs.CommandHandler;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using MongoRepository;
using NSubstitute;

namespace AT.SampleApp.Cqrs.Test.Unit
{
    [TestClass]
    public class ChangeTaskStatusCommandHandlerTest
    {
        private IRepository<Task> _taskRepository;
        private ChangeTaskStatusCommandHandler _handler;

        private const string ID = "000000000000000000000001";
        private const string TITLE = "Blog about MVC with CQRS";

        [TestInitialize]
        public void TestInitialize()
        {
            _taskRepository = Substitute.For<IRepository<Task>>();
            _taskRepository.All().Returns(GetTasks().AsQueryable());
            _handler = new ChangeTaskStatusCommandHandler(_taskRepository);
        }

        public IEnumerable<Task> GetTasks()
        {
            yield return new Task
                             {
                                 Id = ID,
                                 Title = TITLE,
                                 IsCompleted = false,
                                 LastUpdated = new DateTime(2013, 5, 10, 2, 2, 2)
                             };
        }

        [TestMethod]
        public void Execute_ChangeTaskStatus_StatusChanges()
        {
            // Arrange
            var command = new ChangeTaskStatusCommand{TaskId = ID, IsCompleted = true, UpdatedOn = new DateTime(2013, 5, 21, 2, 2, 2)};

            // Act
            _handler.Execute(command);

            // Assert
            _taskRepository.Received().Update(Arg.Is<Task>(x => x.IsCompleted == command.IsCompleted));
        }

        [TestMethod]
        public void Execute_ChangeTaskStatus_LastUpdatedChanges()
        {
            // Arrange
            var command = new ChangeTaskStatusCommand { TaskId = ID, IsCompleted = true, UpdatedOn = new DateTime(2013, 5, 21, 2, 2, 2) };

            // Act
            _handler.Execute(command);

            // Assert
            _taskRepository.Received().Update(Arg.Is<Task>(x => x.LastUpdated == command.UpdatedOn));
        }

        [TestMethod]
        public void Execute_ChangeTaskStatus_TitleDoesntChange()
        {
            // Arrange
            var command = new ChangeTaskStatusCommand { TaskId = ID, IsCompleted = true, UpdatedOn = new DateTime(2013, 5, 21, 2, 2, 2) };

            // Act
            _handler.Execute(command);

            // Assert
            _taskRepository.Received().Update(Arg.Is<Task>(x => x.Title == TITLE));
        }
    }
}
