﻿using System;
using System.Linq;
using System.Collections.Generic;
using AT.SampleApp.Cqrs.Aggregate;
using AT.SampleApp.Cqrs.Query;
using AT.SampleApp.Cqrs.QueryHandler;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using MongoRepository;
using NSubstitute;

namespace AT.SampleApp.Cqrs.Test.Unit
{
    [TestClass]
    public class TasksByStatusQueryHandlerTest
    {
        private IRepository<Task> _taskRepository;
        private TasksByStatusQueryHandler _handler;

        [TestInitialize]
        public void TestInitialize()
        {
            _taskRepository = Substitute.For<IRepository<Task>>();
            _taskRepository.All().Returns(GetTasks().AsQueryable());
            _handler = new TasksByStatusQueryHandler(_taskRepository);
        }

        public IEnumerable<Task> GetTasks()
        {
            yield return new Task
            {
                Id = "000000000000000000000000",
                Title = "Blog about MVC with CQRS",
                IsCompleted = true,
                LastUpdated = new DateTime(2013,5,21,2,2,2)
            };
            yield return new Task
            {
                Id = "000000000000000000000001",
                Title = "Feed the cat",
                IsCompleted = false,
                LastUpdated = new DateTime(2013, 5, 20, 2, 2, 2)
            };
            yield return new Task
            {
                Id = "000000000000000000000002",
                Title = "Get a cat",
                IsCompleted = false,
                LastUpdated = new DateTime(2013, 5, 19, 2, 2, 2)
            };
            yield return new Task
            {
                Id = "000000000000000000000003",
                Title = "Count the stars",
                IsCompleted = true,
                LastUpdated = new DateTime(2013, 5, 19, 1, 1, 1)
            };
            yield return new Task
            {
                Id = "000000000000000000000004",
                Title = "Eat!",
                IsCompleted = true,
                LastUpdated = new DateTime(2013, 5, 18, 5, 1, 1)
            };
        }

        [TestMethod]
        public void Retrieve_LastUpdatedForAllTasksCompletedAndNot_CalculatedCorretly()
        {
            // Arrange
            var query = new TasksByStatusQuery{ IsCompleted = false };

            // Act
            var result = _handler.Retrieve(query);

            // Assert
            Assert.AreEqual(new DateTime(2013, 5, 21, 2, 2, 2), result.LastUpdateForAnyTask);
        }

        [TestMethod]
        public void Retrieve_CompleteTasks_RightCount()
        {
            // Arrange
            var query = new TasksByStatusQuery { IsCompleted = true };

            // Act
            var result = _handler.Retrieve(query);

            // Assert
            Assert.AreEqual(3, result.Tasks.Count());
        }

        [TestMethod]
        public void Retrieve_IncompleteTasks_RightCount()
        {
            // Arrange
            var query = new TasksByStatusQuery { IsCompleted = false };

            // Act
            var result = _handler.Retrieve(query);

            // Assert
            Assert.AreEqual(2, result.Tasks.Count());
        }

    }
}
