﻿using System;
using System.Linq;
using AT.Core.Cqrs;
using AT.SampleApp.Cqrs.Aggregate;
using AT.SampleApp.Cqrs.Command;
using MongoRepository;

namespace AT.SampleApp.Cqrs.CommandHandler
{
    public class ChangeTaskStatusCommandHandler : ICommandHandler<ChangeTaskStatusCommand>
    {
        private readonly IRepository<Task> _taskRepository;

        public ChangeTaskStatusCommandHandler(IRepository<Task> taskRepository)
        {
            if (taskRepository == null) { throw new ArgumentNullException("taskRepository"); }
            _taskRepository = taskRepository;
        }

        public void Execute(ChangeTaskStatusCommand command)
        {
            if (command == null) { throw new ArgumentNullException("command"); }
            if (string.IsNullOrWhiteSpace(command.TaskId)) { throw new ArgumentException("Id is not specified", "command"); }

            var task = _taskRepository.All().Single(x => x.Id == command.TaskId);
            task.IsCompleted = command.IsCompleted;
            task.LastUpdated = command.UpdatedOn;

            _taskRepository.Update(task);
        }

    }
}
