﻿using System.Net;
using System.Web.Mvc;
using AT.Core.Cqrs;
using AT.SampleApp.Cqrs.Command;
using AT.SampleApp.Cqrs.Query;
using AT.SampleApp.Cqrs.QueryResult;

namespace AT.Web.Website.Controllers
{
    public class TaskController : Controller
    {
        private readonly IQueryDispatcher _queryDispatcher;
        private readonly ICommandDispatcher _commandDispatcher;

        public TaskController(IQueryDispatcher queryDispatcher, ICommandDispatcher commandDispatcher)
        {
            _queryDispatcher = queryDispatcher;
            _commandDispatcher = commandDispatcher;
        }

        [HttpGet]
        public ActionResult Index()
        {
            return View();
        }

        [HttpGet]
        public ActionResult TaskList(TasksByStatusQuery query)
        {
            var result = _queryDispatcher.Dispatch<TasksByStatusQuery, TasksByStatusQueryResult>(query);
            return PartialView(result);
        }

        [HttpPost]
        public ActionResult ChangeTaskStatus(ChangeTaskStatusCommand command)
        {
            _commandDispatcher.Dispatch(command);
            return new HttpStatusCodeResult(HttpStatusCode.Accepted);
        }
    }
}
